import operator
import numpy as np

def get_includes_dict(data2d,cutoff=8):
    """
    Determines which control patterns make up at least a certain percent
    of the total control (defined by `cutoff`) for each point in a
    parameter scan.

    Returns a dictionary with parameter scan points as keys and lists of control
    pattern names as values.
    """
    includes_dict = {}
    for point in xrange(data2d.scan_results.scan_points):
        CP_per_dict = {k:v for k,v in zip(data2d.scan_results.scan_out,data2d.scan_results.scan_results[point,:])}
        sorted_CP_per_dict = sorted(CP_per_dict.items(), key=operator.itemgetter(1))

        includes = []

        for k,v in reversed(sorted_CP_per_dict):
            if v > cutoff:
                includes.append(k)


        includes_dict[data2d.scan_results.scan_range[point]] = sorted(includes)
    return includes_dict

def temporal_cutoff(includes_dict,min_cut):
    """
    For a dictionary of scanned points as keys and control patterns as values,
    determines which control patterns appear in the dictionary for a percentage
    of points as defined by a cutoff (min_cut).

    Returns a dictionary with the same structure as the argument `includes_dict`
    but with only the control patterns that appear for at least the percentage
    of points (keys) as defined by the cutoff.
    """
    scan_points = len(includes_dict)
    counts_dict = {}
    for k,v in sorted(includes_dict.iteritems()):
        for cp in v:
            if cp in counts_dict:
                counts_dict[cp] += 1
            else:
                counts_dict[cp] = 1
    must_exceed = scan_points/100. * min_cut
    keepers = [k for k,v in counts_dict.iteritems() if v >= must_exceed]

    new_includes_dict = {}
    for k, v in sorted(includes_dict.iteritems()):
        new_v = []
        for cp in v:
            if cp in keepers:
                new_v.append(cp)
        new_includes_dict[k] = new_v
    return new_includes_dict

def get_important_cps(includes_dict):
    """Returns a list of all the important control patterns"""
    all_relevent = []
    for each in includes_dict.values():
        all_relevent = all_relevent + each
    return list(set(all_relevent))

def get_total_cp_contribution(control_patterns, data2d):
    CP_vals = []
    for CP in control_patterns:
        ind = data2d.scan_results.scan_out.index(CP)
        CP_vals.append(data2d.scan_results.scan_results[:,ind])
    CP_vals = np.vstack(CP_vals).transpose()
    total_contrib_new = np.vstack([data2d.scan_results.scan_range,
                                   np.sum(CP_vals,1)]).transpose()
    return total_contrib_new

def find_category_ranges(includes_dict):
    """find ranges of the scan range which are dominated by the same CPs
    start and end points of ranges are collected together with the
    control patterns in a dictionary with numbers as keys (this may just
    as well be a list, but this is legacy and works fine)"""
    category_ranges = {}
    current_cat_name = 1
    first = True
    last_key_index = len(includes_dict) - 1
    key_index = 0
    for k,v in sorted(includes_dict.iteritems()):
        next_cat = strip_chars(str(v),' []\'')
        if first:
            first=False
            start = k
            current_cat = next_cat
        elif next_cat != current_cat:
            end = prev_k
            category_ranges[current_cat_name] = (start,end,list(current_cat.split(',')))
            start = k
            current_cat = next_cat
            current_cat_name += 1
        elif key_index == last_key_index:
            end = k
            category_ranges[current_cat_name] = (start,end,list(current_cat.split(',')))
        key_index += 1
        prev_k = k
    return category_ranges

def strip_chars(string,chars):
    """
    Removes multiple characters from a string.

    Parameters
    ----------
    string: str
        A string with unwanted characters
    chars: str or list
        A string or list containing the characters
        to remove from `string`

    Returns
    -------
    str
        A string without any unwanted characters
    """
    new_str = string
    for each in chars:
        new_str = new_str.replace(each,'')
    return new_str

def convert_color(r,b,g,a=1):
    return r/255.,b/255.,g/255.,a
